package net.nevet5gi.buzzbot;

import org.simpleyaml.configuration.file.YamlFile;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;

public class Config {
    private static final YamlFile YML_FILE = new YamlFile("./config.yml");
    private static final Logger LOGGER = LoggerFactory.getLogger(Config.class);

    public static void loadConfig() {
        YML_FILE.setConfigurationFile("./config.yml");
        try {
            if (!YML_FILE.exists()) {
                LOGGER.info("Config file not found, creating new one...");
                YML_FILE.createNewFile(true);
                LOGGER.info("Config file created!");
            } else {
                LOGGER.info("Loading Config file...");
                YML_FILE.loadWithComments();
                LOGGER.info("Config file loaded!");
            }
        } catch (final Exception e) {
            LOGGER.error("Error while loading config file!");
            e.printStackTrace();
        }

        
        YML_FILE.addDefault("bot.token", "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx");
        YML_FILE.addDefault("bot.prefix", "!");
        YML_FILE.addDefault("bot.activity", "playing");
        YML_FILE.addDefault("bot.action", "with myself");
        YML_FILE.addDefault("database.url", "localhost");
        YML_FILE.addDefault("database.database", "buzzbot");
        YML_FILE.addDefault("database.user", "barry");
        YML_FILE.addDefault("database.password", "benson");
        YML_FILE.addDefault("filter.path.mild", "./mild.txt");
        YML_FILE.addDefault("filter.path.moderate", "./moderate.txt");
        YML_FILE.addDefault("filter.path.strong", "./strong.txt");

        try {
            YML_FILE.save();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static YamlFile getConfig() {
        return YML_FILE;
    }
}
