package net.nevet5gi.buzzbot.commands;

import net.dv8tion.jda.api.EmbedBuilder;
import net.dv8tion.jda.api.entities.TextChannel;
import net.dv8tion.jda.api.interactions.commands.OptionMapping;
import net.dv8tion.jda.api.interactions.commands.OptionType;
import net.dv8tion.jda.api.interactions.commands.build.CommandData;
import net.nevet5gi.buzzbot.Config;
import net.nevet5gi.buzzbot.commands.utils.*;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class HelpCmd implements ICommand, ISlashCommand {

    private final CommandManager manager;

    public HelpCmd(CommandManager manager) {
        this.manager = manager;
    }

    @Override
    public void handle(CommandContext ctx) {
        List<String> args = ctx.getArgs();
        TextChannel channel = ctx.getChannel();

        if (args.isEmpty()) {
            StringBuilder sb = new StringBuilder();

            EmbedBuilder eb = new EmbedBuilder();

            eb.setTitle("List of Commands");

            manager.getCommands().stream().map(ICmdGeneric::getName).forEach(
                    (it) -> sb.append('`').append(Config.getConfig().getString("bot.prefix")).append(it).append("`\n")
            );

            eb.addField("", sb.toString(), false);
            channel.sendMessageEmbeds(eb.build()).queue();

            return;
        }

        String search = args.get(0);
        ICommand command = manager.getCommand(search);
        ISlashCommand slashCommand = manager.getSlashCommand(search);

        if (command == null && slashCommand == null) {
            channel.sendMessage("Nothing found for " + search).queue();
            return;
        }

        channel.sendMessage(command.getHelp()).queue();
    }

    @Override
    public void handleSlash(CommandContext ctx) {
        StringBuilder sb = new StringBuilder();
        EmbedBuilder eb = new EmbedBuilder();

        OptionMapping option = ctx.getSlashEvent().getOption("command");
        String cmd = null;
        if (option != null) cmd = option.getAsString();

        if (cmd == null) {
            ICommand command = manager.getCommand(ctx.getSlashEvent().getName());
            ISlashCommand slashCommand = manager.getSlashCommand(ctx.getSlashEvent().getName());

            if (command == null && slashCommand == null) {
                ctx.getSlashEvent().getHook().sendMessage("Nothing found for " + ctx.getSlashEvent()).queue();
                return;
            }

            eb.setTitle("Commands");

            manager.getCommands().stream().map(ICmdGeneric::getName).forEach(
                    (it) -> sb.append('`').append(Config.getConfig().getString("bot.prefix")).append(it).append("`\n")
            );

            eb.addField("", sb.toString(), false);

        } else {
            ICommand scmd = manager.getCommand(cmd);

            if (scmd != null) {
                eb.setTitle(scmd.getName());
                eb.addField("", scmd.getHelp(), false);
            } else {
                eb.setTitle("Error");
                eb.addField("", "Unknown command, run `/help` to see a list of available commands", false);
            }
        }

        ctx.getSlashEvent().getHook().sendMessageEmbeds(eb.build()).queue();
    }

    @Override
    public String getName() {
        return "help";
    }

    @Override
    public String getDescription() {
        return "Lists bot commands";
    }

    @Override
    public @NotNull CommandData getCommandData() {

        return new CommandData(this.getName(), this.getDescription())
                .addOption(OptionType.STRING, "command", "Gives info on this command", false);
    }

    @Override
    public String getHelp() {
        return "Shows the list of bot commands\n" +
                "Usage: `" + Config.getConfig().getString("bot.prefix") + "help [command]`";
    }

    @Override
    public List<String> getAliases() {
        return List.of("commands", "cmds", "commandlist");
    }
}
