package net.nevet5gi.buzzbot.functions;

import net.dv8tion.jda.api.events.message.guild.GuildMessageReceivedEvent;
import net.dv8tion.jda.api.events.message.guild.GuildMessageUpdateEvent;
import net.nevet5gi.buzzbot.Config;
import net.nevet5gi.buzzbot.util.Dictionary;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class ProfanityFilter {
    private final Dictionary mildDictionary;
    private final Dictionary moderateDictionary;
    private final Dictionary strongDictionary;

    public ProfanityFilter() {
        mildDictionary = new Dictionary();
        moderateDictionary = new Dictionary();
        strongDictionary = new Dictionary();
    }

    public void handle(GuildMessageReceivedEvent event) {
        event.getMessage().delete().queue();
        event.getChannel().sendMessage("No swearing please :)").queue();
    }

    public void handleEdit(GuildMessageUpdateEvent event) {
        event.getMessage().delete().queue();
        event.getChannel().sendMessage("Nice try ;)").queue();
    }

    public void buildDictionaries() {
        String mildPath = Config.getConfig().getString("filter.path.mild");
        String moderatePath = Config.getConfig().getString("filter.path.moderate");
        String strongPath = Config.getConfig().getString("filter.path.strong");

        String line;
        BufferedReader in = null;

        try {
            in = new BufferedReader(new FileReader(mildPath));
            while ((line = in.readLine()) != null) {
                mildDictionary.addToDictionary(line);
                moderateDictionary.addToDictionary(line);
                strongDictionary.addToDictionary(line);
            }

            in = new BufferedReader(new FileReader(moderatePath));
            while ((line = in.readLine()) != null) {
                moderateDictionary.addToDictionary(line);
                strongDictionary.addToDictionary(line);
            }

            in = new BufferedReader(new FileReader(strongPath));
            while ((line = in.readLine()) != null) {
                strongDictionary.addToDictionary(line);
            }

        } catch (FileNotFoundException e) { // FileReader
            e.printStackTrace();
        } catch (IOException e) { // readLine
            e.printStackTrace();
        } finally {
            try {
                in.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    public boolean containsProfanity(String input, String strength) {
        Dictionary dictionary = null;

        switch (strength) {
            case "mild" -> dictionary = mildDictionary;
            case "moderate" -> dictionary = moderateDictionary;
            case "strong" -> dictionary = strongDictionary;
        }

        if (dictionary == null) {
            dictionary = new Dictionary();
        }

        input = convert(input);

        for (String profanity : dictionary.getDictionary()) {
            String pattern = "\\b" + Pattern.quote(profanity) + "\\b";
            Pattern p = Pattern.compile(pattern);
            Matcher m = p.matcher(input);
            if (m.find()) {
                return true;
            }
        }
        return false;
    }

    public String convert(String input) {
        return input.toLowerCase()
                .replace("@", "a")
                .replace("$", "s")
                .replace("!", "i")
                .replace("(", "c");
    }
}
